--- old/usr/src/lib/smbsrv/libsmb/common/smb_cfg.c	2020-04-15 01:43:36.213503298 +0000
+++ new/usr/src/lib/smbsrv/libsmb/common/smb_cfg.c	2020-04-15 01:43:36.219039645 +0000
@@ -21,6 +21,7 @@
 /*
  * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  * Copyright 2018 Nexenta Systems, Inc.  All rights reserved.
+ * Copyright 2020 RackTop Systems, Inc.
  */
 
 /*
@@ -150,6 +151,7 @@
 	{SMB_CI_MIN_PROTOCOL, "min_protocol", SCF_TYPE_ASTRING, 0},
 	{SMB_CI_BYPASS_TRAVERSE_CHECKING,
 	    "bypass_traverse_checking", SCF_TYPE_BOOLEAN, 0},
+	{SMB_CI_ENCRYPT_CIPHER, "encrypt_cipher", SCF_TYPE_ASTRING, 0},
 
 	/* SMB_CI_MAX */
 };
@@ -165,6 +167,7 @@
  */
 static struct str_val
 smb_versions[] = {
+	{ "3.11",	SMB_VERS_3_11 },
 	{ "3.02",	SMB_VERS_3_02 },
 	{ "3.0",	SMB_VERS_3_0 },
 	{ "2.1",	SMB_VERS_2_1 },
@@ -173,6 +176,16 @@
 	{ NULL,		0 }
 };
 
+/*
+ * Supported encryption ciphers.
+ */
+static struct str_val
+smb31_encrypt_ciphers[] = {
+	{ "aes128-ccm",	SMB3_CIPHER_AES128_CCM },	/* SMB 3.x */
+	{ "aes128-gcm",	SMB3_CIPHER_AES128_GCM },	/* SMB 3.1.1 */
+	{ NULL,		0 }
+};
+
 static smb_cfg_param_t *smb_config_getent(smb_cfg_id_t);
 
 static boolean_t smb_is_base64(unsigned char c);
@@ -1224,7 +1237,7 @@
  * whole range that we implement). For that reason, this should usually be the
  * highest protocol version we implement.
  */
-uint32_t max_protocol_default = SMB_VERS_3_02;
+uint32_t max_protocol_default = SMB_VERS_3_11;
 
 uint32_t
 smb_config_get_max_protocol(void)
@@ -1263,6 +1276,34 @@
 }
 
 /*
+ * Only SMB 3.x supports encryption.
+ * SMB 3.0.2 uses AES128-CCM only.
+ * SMB 3.1.1 - AES128-CCM or AES128-GCM.
+ */
+uint16_t
+smb31_config_get_encrypt_cipher(void)
+{
+	uint32_t max_proto = smb_config_get_max_protocol();
+	uint16_t cipher = SMB3_CIPHER_AES128_GCM; /* by default AES128-GCM */
+	char str[12];
+	int i;
+
+	if (max_proto < SMB_VERS_3_11)
+		return (SMB3_CIPHER_NONE);
+
+	/* SMB 3.1.1 */
+	if (smb_config_getstr(SMB_CI_ENCRYPT_CIPHER, str, sizeof (str))
+	    == SMBD_SMF_OK) {
+		for (i = 0; smb31_encrypt_ciphers[i].str != NULL; i++) {
+			if (strcmp(str, smb31_encrypt_ciphers[i].str) == 0)
+				cipher = smb31_encrypt_ciphers[i].val;
+		}
+	}
+
+	return (cipher);
+}
+
+/*
  * If smb2_enable is present and max_protocol is empty,
  * set max_protocol.  Delete smb2_enable.
  */
